--		Zaglavlje fajla --------------------------------------------------------
--       Naziv mux2xn - Dvoulazni n-bitni multiplekser (2xn ->n)
--   		 Opis Propusta jedan od dva ulazna vektora od n bita na izlaz,
--				   zavisno od vrednosti ulaznog signala s (select).
--        Fajl C:\My Documents\etf\ri5vlsi\domaci\mk2002\sabirac\vhdl\sabirac\
--					SRC\mux2xn.VHD
--		  Namena Koristi se kao blok u 5bit sabiracu sa uslovnim sumama
--    Napomene U realnosti kasnjenje kroz and,or,xor i xnor kola nije isto,ali
--					 je ta pretpostavka uvedena u postavci zadatka.
--					U slucaju dovodjenja 'U','X','W' i '-' na s port, na izlaz se
--					 prosledjuje vektor X-ova, a u slucaju 'Z' - vektor Z-ova.
-- Ogranicenja Kasnjenje kroz logicko kolo - Tk i sirina vektora - n se moraju
--					precizirati pri pravljenju instance ove komponente na visem
--					nivou, u generic mapi.
--      Greske -
--   Korisceni
--  	  paketi IEEE.std_logic_1164
--       Autor Milos Kovacevic, pirate@bitsyu.net, http://pirate.jump.to
--   Korisceni
--	  simulator Active VHDL 3.2 build 207
--     Verzija 1.02 (02/02/2002)
--      Izmene Izmene u arhitekturi
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

entity mux2xn is
--		Zaglavlje entiteta ------------------------------------------------------
--         Opis Entitet dvoulaznog n-bitnog multipleksera, sadrzi dva ulazna,
--					 n-bitna porta, port selekcije, i izlazni n-bitni port.
--  Ogranicenja Tk-vreme kasnjenja, i n-sirina ulaza se moraju precizirati.
-- Pretpostavke Tk je isto za sva logicka kola.
--      Verzija 1.00
--       Izmene -
--        Autor Milos Kovacevic, pirate@bitsyu.net, http://pirate.jump.to
--        Datum 29/01/2002
-------------------------------------------------------------------------------
	generic (
		n:positive;			-- sirina ulaznih i izlaznog signala, u bitima
		Tk:time				-- vreme kasnjenja jednog logickog kola
	);
   port (
      A: in  STD_LOGIC_VECTOR (n-1 downto 0); -- 0. ulazni n-bitni signal
      B: in  STD_LOGIC_VECTOR (n-1 downto 0); -- 1. ulazni n-bitni signal
      s: in  STD_LOGIC;								 -- signal selekcije ulaza

      Y: out STD_LOGIC_VECTOR (n-1 downto 0)  -- n-bitni izlazni signal
    );
end mux2xn;

architecture mux2xnbeh of mux2xn is
--		Zaglavlje arhitekture ---------------------------------------------------
--         Opis Arhitektura ponasanja dvoulaznog n-bitnog multipleksera, vrlo
--					 pazljivo projektovana i testirana.
--  Ogranicenja -
-- Pretpostavke Kasnjenje je isto za and, not, or i nor logicka kola.
--      Verzija 1.02
--       Izmene U slucaju dovodjenja visoke impedanse na ulaz za selekciju, na
--					 izlaz se prosledjuje vektor Z-ova, a u slucaju 'U','X','W'i'-'
--					 na izlaz se prosledjuje vektor X-ova.
--      Verzija 1.01
--       Izmene Izbacen proces i ubacene uslovne dodele vrednosti.
--        Autor Milos Kovacevic, pirate@bitsyu.net, http://pirate.jump.to
--        Datum 02/02/2002
-------------------------------------------------------------------------------
	-- vektor visoke impedanse (Z-ova):
	constant Z: std_logic_vector(A'range) := (others=>'Z');
	-- vektor 0 ili 1 (X-ova):
	constant X: std_logic_vector(A'range) := (others=>'X');
	-- broj nivoa logickih kola od ulaza do izlaza, za odredjivanje kasnjenja
	constant l: positive :=3;
begin
	with s select	-- ovo je selektivna dodela vrednosti
		Y <= 			-- aktivira se automatski pri promeni A, B ili s
			A after l*Tk when '0',
			A after l*Tk when 'L', -- slaba 0 se tretira kao 0			
			B after l*Tk when '1',
			B after l*Tk when 'H', -- slaba 1 se tretira kao 1
			Z after l*Tk when 'Z', -- Z prosledjuje na izlaz vektor Z
			X after l*Tk when others;
end architecture mux2xnbeh;
